#version 430 core

layout(location = 0) out vec3 Color;
layout(location = 1) out vec3 Glow;

layout(binding = 0) uniform sampler2D albedo; //16
layout(binding = 1) uniform sampler2D normal; //16
layout(binding = 2) uniform sampler2D glow; //8
layout(binding = 3) uniform samplerCube depthMap; // 6 * 32

in VS_OUT
{
	vec3 WorldPosition;
	vec3 WorldLight;
	vec3 TangentPosition;
	vec3 TangentLight;
	vec3 TangentView;
	vec2 UV;
} input;

uniform vec2 glowLimits;
uniform vec3 lightIntensity;
uniform vec3 lightAttenuation;
uniform bool isEmitter;

void drawEmitter(void) 
{
	Color = lightIntensity;
	Glow = lightIntensity;
}

bool inShadow(void)
{
	vec3 lightVector = input.WorldPosition - input.WorldLight;
	float closestDepth = texture(depthMap, lightVector).r * (25.f - .01f) + .01f;
	float depth = length(lightVector);
	//Color = vec3(closestDepth / 25.f);
	return depth - 0.15f > closestDepth? true : false;
}

float lighting(void)
{
	vec3 L = input.TangentLight - input.TangentPosition;
	float att = length(L);
	
	vec3 N = normalize(texture(normal, input.UV).rgb * 2.0 - vec3(1));
	vec3 V = normalize(input.TangentView - input.TangentPosition);
	vec3 H = normalize(L + V);
	L = normalize(L);
	
	att = 1.f / (lightAttenuation.x + lightAttenuation.y * att + lightAttenuation.z * att * att);
	float diffuse = max(dot(N, L), 0);
	float specular = 0;
	
	if( diffuse > 0.001f)
		specular = pow(max(dot(N, H), 0), 32);
		
	return (diffuse + specular) * att;
}

void shadeObject(void) 
{
	vec3 color = texture(albedo, input.UV).rgb;
	vec3 material = texture(glow, input.UV).rgb;
	
	bool shadow = inShadow();
	vec3 illumination = vec3(0.05f);
	
	if(!shadow)
		illumination += lighting() * lightIntensity;
	
	Color =  color * illumination;
	
	float clampedValue = clamp(input.UV.y, glowLimits.x, glowLimits.y);
	if(abs(input.UV.y - clampedValue) < 0.01f)
		Glow = vec3(0.8117f, 0.0627,0.1254) * material.r;
	else
		Glow = vec3(0);
}

void main(void)
{
	if(isEmitter)
		drawEmitter();
	else
		shadeObject();
}