#version 430 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec3 tangent;
layout(location = 3) in vec2 uv;

out VS_OUT
{
	vec3 WorldPosition;
	vec3 WorldLight;
	vec3 TangentPosition;
	vec3 TangentLight;
	vec3 TangentView;
	vec2 UV;
} vertOut;

uniform bool isEmitter;

uniform vec3 cameraPosition;
uniform vec3 lightPosition;

uniform mat3 normalMatrix;
uniform mat4 modelMatrix;
uniform mat4 viewProjectionMatrix;



void normalMappedWorld(void)
{
	vec4 P = modelMatrix * vec4(position, 1);
	vec3 N = normalize(normalMatrix * normal);
	vec3 T = normalize(normalMatrix * tangent);
	T = normalize(T - dot(T, N) * N);
	vec3 B = cross(N, T);
	
	vertOut.WorldPosition = P.xyz;
	vertOut.WorldLight = lightPosition;
	vertOut.TangentPosition = vec3(dot(P.xyz, T), dot(P.xyz, B), dot(P.xyz, N));
	vertOut.TangentLight = vec3(dot(lightPosition, T), dot(lightPosition, B), dot(lightPosition, N)); // = transpose(TBN) * L
	vertOut.TangentView = vec3(dot(cameraPosition, T), dot(cameraPosition, B), dot(cameraPosition, N));
	vertOut.UV = uv;
	
	gl_Position = viewProjectionMatrix * P;
}

void shadeEmitter(void)
{
	gl_Position = viewProjectionMatrix * modelMatrix * vec4(position, 1);
}

void main(void)
{
	if(isEmitter)
		shadeEmitter();
	else
		normalMappedWorld();
}