#version 430 core

/*
layout(location = 0) out vec4 Color;
layout(location = 1) out vec4 Glow;
layout(location = 2) out vec4 Position;

in vec2 textCoords;
in vec4 textPosition;

uniform sampler2D map;

void main(void) {
	Color = texture(map, textCoords);
	Position = textPosition;
	if(Color.x > 0.5f)
		Glow = vec4(1, 0, 0, 1);
	else
		Glow = vec4(0, 0, 0, 1);
	
	//Color = vec4(1, 1, 0, 1);
}
*/

layout(location = 0) out vec3 Color;
layout(location = 1) out vec3 Glow;

in vec2 verTexcoord;
in vec3 verNormal;
in vec3 verPosition;

uniform vec3 cameraPosition;
uniform vec3 lightPosition;
uniform vec3 lightColor;

layout(binding = 0) uniform sampler2D albedo;
layout(binding = 2) uniform sampler2D material;

vec3 lighting(float spec) {
	float diffuse = 0;
	float specular = 0;
	
	vec3 N = normalize(verNormal);
	vec3 L = normalize(lightPosition - verPosition);
	diffuse = max(dot(N, L), 0);
	
	if(diffuse > 0) {
		vec3 R = reflect(-L, N);
		vec3 V = normalize(cameraPosition - verPosition);
		specular = max(dot(R,V), 0);
		
		specular = pow(specular, 128);
			
		specular *= spec;
	}
	
	return (0.15f + diffuse + specular) * lightColor;
}

void main(void) {

	vec3 alb = texture(albedo, verTexcoord).rgb;
	vec3 gss = texture(material, verTexcoord).rgb;
	
	Color = lighting(gss.g) * alb;
	Glow = alb * gss.r;
}
